import { _decorator, Camera, Component, EventTouch, geometry, input, Input, instantiate, Label, mat4, Mat4, MeshRenderer, Node, Prefab, Quat, Vec3, Vec4 } from 'cc';
import { meshTouchCtrl } from './meshTouchCtrl';
const { ccclass, property } = _decorator;

const v3_0 = new Vec3();
const v3_1 = new Vec3();
const v4_0 = new Vec4();
const m4_0 = new Mat4();

const q_0 = new Quat();

@ccclass('meshTouchExample')
export class meshTouchExample extends Component {

    @property(Camera)
    readonly cameraCom!: Camera;

    @property(Node)
    debug: Node = null;

    @property(Label)
    debugText: Label = null;

    meshes: MeshRenderer[];

    private _ray: geometry.Ray = new geometry.Ray();
    private _mesh: MeshRenderer;

    onEnable() {
        input.on(Input.EventType.TOUCH_START, this.onTouchStart, this);
        this.meshes = this.node.getComponentsInChildren(MeshRenderer);
    }

    onDisable() {
        input.off(Input.EventType.TOUCH_START, this.onTouchStart, this);
    }

    onTouchStart(event: EventTouch) {
        const touch = event.touch!;
        this.cameraCom.screenPointToRay(touch.getLocationX(), touch.getLocationY(), this._ray);
        this.rayHit();

    }

    /* check model hit */
    rayHit() {
        let distance = 300;
        let mesh: MeshRenderer
        for (let v of this.meshes) {
            let dis = geometry.intersect.rayModel(this._ray, v.model);
            if (dis && dis < distance) {
                distance = dis;
                mesh = v;
            }
        }

        if (mesh) {
            this._ray.computeHit(v3_0, distance);

            const node = mesh.node;
            const halfSize = mesh.model.modelBounds.halfExtents;

            /* set debug node pos and scale */
            this.debug.setWorldPosition(v3_0);
            const scale = halfSize.x * 0.1 * node.scale.x;
            this.debug.setScale(scale, scale, scale);

            /* convert world pos to a node's local pos by invert the node's worldmat and mul with the world pos */
            m4_0.set(node.worldMatrix);
            m4_0.invert();
            Vec3.transformMat4(v3_0, v3_0, m4_0)

            if (halfSize.y == 0) {
                /* the plane */
                const x = halfSize.x;
                const z = halfSize.z;
                v4_0.x = (x + v3_0.x) / (x * 2);
                v4_0.y = (z + v3_0.z) / (z * 2);
            } else {
                /* the quad */
                const x = halfSize.x;
                const y = halfSize.y;
                v4_0.x = (x + v3_0.x) / (x * 2);
                v4_0.y = (y - v3_0.y) / (y * 2);
            }

            v4_0.w = 0.1;

            /* get the script in mesh node */

            const meshCtrl = node.getComponent(meshTouchCtrl);

            meshCtrl && meshCtrl.waveProp.set(v4_0);

            /* debug text */
            this.debugText.string = "Clicked: " + node.name + "  UV: " + v4_0.x.toFixed(2) + ", " + v4_0.y.toFixed(2);

        }
    }



}

