import { _decorator, Component, EventTouch, Label, Node, renderer, Sprite, UITransform, Vec2, Vec3, Vec4 } from 'cc';
const { ccclass, property } = _decorator;

const v2_0 = new Vec2();

const v3_0 = new Vec3();

const v4_0 = new Vec4();


@ccclass('spTouchExample')
export class spTouchExample extends Component {

    @property
    waveDis = 0.4;
    @property
    waveSpeed = 1;
    @property
    waveStr = 0.5;
    @property(Node)
    debug: Node = null;

    @property(Label)
    debugText: Label = null;

    public waveProp: Vec4 = new Vec4();

    private _trans: UITransform;
    private _pass: renderer.Pass;
    private _handle: number;

    start() {
        this._trans = this.node.getComponent(UITransform);

        this.waveProp.w = 100;

        this.waveProp.z = this.waveStr;

        this._pass = this.node.getComponent(Sprite).material.passes[0];

        /* get the handle of FBO in advance */

        this._handle = this._pass.getHandle("waveFactor");

    }

    onEnable() {

        this.node.on(Node.EventType.TOUCH_START, this.onTouchStart, this);
    }

    onDisable() {

        this.node.off(Node.EventType.TOUCH_START, this.onTouchStart, this);

    }

    onTouchStart(event: EventTouch) {

        const touch = event.touch;

        touch.getUILocation(v2_0);

        v3_0.set(v2_0.x, v2_0.y);

        /* convert the wolrd ui pos to node's local pos */

        this._trans.convertToNodeSpaceAR(v3_0, v3_0);

        this.debug.setPosition(v3_0);

        /* cal the UV in sprite */

        const size = this._trans.contentSize;

        const x = size.x;

        const y = size.y;

        v4_0.x = (x * 0.5 + v3_0.x) / x;

        v4_0.y = 1 - (y * 0.5 + v3_0.y) / y;

        v4_0.w = 0;

        this.waveProp.set(v4_0);

        this.debugText.string = "Clicked: " + this.node.name + "  UV: " + v4_0.x.toFixed(2) + ", " + v4_0.y.toFixed(2);

    }


    update(dt) {

        if (this.waveProp.w < 100) {

            this.waveProp.w += dt * this.waveSpeed;
            if (this.waveProp.w > this.waveDis) {
                this.waveProp.w = 100;
            }
            /* since the handle has been got in advance, which can be recon in a frequent update case*/
            this._pass.setUniform(this._handle, this.waveProp);

        }

    }


}

